@@ -1,4 +1,4 @@
-/*	$OpenBSD: parser.c,v 1.169 2025/08/19 08:32:24 tb Exp $ */
+/*	$OpenBSD: parser.c,v 1.174 2026/01/12 10:48:20 tb Exp $ */
 /*
  * Copyright (c) 2019 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -589,6 +589,13 @@ proc_parser_cert(char *file, const unsigned char *der, size_t len,
 	if (cert == NULL)
 		goto out;
 
+	if (cert->purpose != CERT_PURPOSE_CA &&
+	    cert->purpose != CERT_PURPOSE_BGPSEC_ROUTER) {
+		warnx("%s: %s not allowed in a manifest", file,
+		    purpose2str(cert->purpose));
+		goto out;
+	}
+
 	a = find_issuer(file, entp->certid, cert->aki, entp->mftaki);
 	if (a == NULL)
 		goto out;
@@ -619,6 +626,10 @@ proc_parser_cert(char *file, const unsigned char *der, size_t len,
 	 * Add validated CA certs to the RPKI auth tree.
 	 */
 	if (cert->purpose == CERT_PURPOSE_CA) {
+		if (sizeof(cert->mfthash) != entp->datasz)
+			errx(1, "%s: corrupted entity", file);
+
+		memcpy(cert->mfthash, entp->data, entp->datasz);
 		auth_insert(file, &auths, cert, a);
 	}
 
@@ -732,46 +743,6 @@ proc_parser_root_cert(struct entity *entp, struct cert **out_cert)
 	}
 }
 
-/*
- * Parse a ghostbuster record
- */
-static struct gbr *
-proc_parser_gbr(char *file, const unsigned char *der, size_t len,
-    const struct entity *entp, X509_STORE_CTX *ctx)
-{
-	struct gbr	*gbr;
-	struct cert	*cert = NULL;
-	struct crl	*crl;
-	struct auth	*a;
-	const char	*errstr;
-
-	if ((gbr = gbr_parse(&cert, file, entp->talid, der, len)) == NULL)
-		goto out;
-
-	a = find_issuer(file, entp->certid, cert->aki, entp->mftaki);
-	if (a == NULL)
-		goto out;
-	crl = crl_get(&crls, a);
-
-	if (!valid_x509(file, ctx, cert->x509, a, crl, &errstr)) {
-		warnx("%s: %s", file, errstr);
-		goto out;
-	}
-
-	gbr->talid = a->cert->talid;
-
-	gbr->expires = x509_find_expires(cert->notafter, a, &crls);
-	cert_free(cert);
-
-	return gbr;
-
- out:
-	gbr_free(gbr);
-	cert_free(cert);
-
-	return NULL;
-}
-
 /*
  * Parse an ASPA object
  */
@@ -889,7 +860,6 @@ parse_entity(struct entityq *q, struct ibufqueue *msgq, X509_STORE_CTX *ctx,
 	struct mft	*mft;
 	struct roa	*roa;
 	struct aspa	*aspa;
-	struct gbr	*gbr;
 	struct tak	*tak;
 	struct spl	*spl;
 	struct ibuf	*b;
@@ -926,7 +896,13 @@ parse_entity(struct entityq *q, struct ibufqueue *msgq, X509_STORE_CTX *ctx,
 			tal_free(tal);
 			break;
 		case RTYPE_CER:
-			if (entp->data != NULL) {
+			/*
+			 * If entp->datasz == SHA256_DIGEST_LENGTH, we have a
+			 * cert added from a manifest, so it is not a root cert.
+			 * proc_parser_cert() will also make sure of this.
+			 */
+			if (entp->data != NULL &&
+			    entp->datasz != SHA256_DIGEST_LENGTH) {
 				file = proc_parser_root_cert(entp, &cert);
 			} else {
 				file = parse_load_file(entp, &f, &flen);
@@ -995,15 +971,6 @@ parse_entity(struct entityq *q, struct ibufqueue *msgq, X509_STORE_CTX *ctx,
 				roa_buffer(b, roa);
 			roa_free(roa);
 			break;
-		case RTYPE_GBR:
-			file = parse_load_file(entp, &f, &flen);
-			io_str_buffer(b, file);
-			gbr = proc_parser_gbr(file, f, flen, entp, ctx);
-			if (gbr != NULL)
-				mtime = gbr->signtime;
-			io_simple_buffer(b, &mtime, sizeof(mtime));
-			gbr_free(gbr);
-			break;
 		case RTYPE_ASPA:
 			file = parse_load_file(entp, &f, &flen);
 			io_str_buffer(b, file);
@@ -1237,10 +1204,6 @@ proc_parser(int fd, int nthreads)
 	if (pledge("stdio rpath", NULL) == -1)
 		err(1, "pledge");
 
-	ERR_load_crypto_strings();
-	OpenSSL_add_all_ciphers();
-	OpenSSL_add_all_digests();
-	x509_init_oid();
 	constraints_parse();
 
 	if ((globalmsgq = ibufq_new()) == NULL)
