@@ -1,4 +1,4 @@
-/*	$OpenBSD: print.c,v 1.69 2025/09/15 11:52:07 job Exp $ */
+/*	$OpenBSD: print.c,v 1.72 2025/12/30 09:04:09 job Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -610,36 +610,6 @@ spl_print(const struct cert *c, const struct spl *s)
 		json_do_end();
 }
 
-void
-gbr_print(const struct cert *c, const struct gbr *p)
-{
-	if (outformats & FORMAT_JSON) {
-		json_do_string("type", "gbr");
-		json_do_string("ski", c->ski);
-		x509_print(c->x509);
-		json_do_string("aki", c->aki);
-		json_do_string("aia", c->aia);
-		json_do_string("sia", c->signedobj);
-		json_do_int("signing_time", p->signtime);
-		json_do_int("valid_since", c->notbefore);
-		json_do_int("valid_until", c->notafter);
-		if (p->expires)
-			json_do_int("expires", p->expires);
-		json_do_string("vcard", p->vcard);
-	} else {
-		printf("Subject key identifier:   %s\n", pretty_key_id(c->ski));
-		x509_print(c->x509);
-		printf("Authority key identifier: %s\n", pretty_key_id(c->aki));
-		printf("Authority info access:    %s\n", c->aia);
-		printf("Subject info access:      %s\n", c->signedobj);
-		printf("Signing time:             %s\n", time2str(p->signtime));
-		printf("GBR not before:           %s\n",
-		    time2str(c->notbefore));
-		printf("GBR not after:            %s\n", time2str(c->notafter));
-		printf("vcard:\n%s", p->vcard);
-	}
-}
-
 void
 rsc_print(const struct cert *c, const struct rsc *p)
 {
@@ -887,24 +857,21 @@ geofeed_print(const struct cert *c, const struct geofeed *p)
 static void
 print_ccr_mftstate(struct ccr *ccr)
 {
-	char *aki, *hash;
+	char *aki, *hash, *ski;
 	struct ccr_mft *ccr_mft;
-
-	if (base64_encode(ccr->mfts_hash, SHA256_DIGEST_LENGTH, &hash) == -1)
-		errx(1, "base64_encode");
+	struct ccr_mft_sub_ski *sub;
 
 	if (outformats & FORMAT_JSON) {
 		json_do_object("manifest_state", 0);
 		json_do_int("most_recent_update", ccr->most_recent_update);
-		json_do_string("hash", hash);
-		json_do_array("refs");
+		json_do_string("hash", ccr->mfts_hash);
+		json_do_array("mft_instances");
 	} else {
-		printf("Manifest state hash:      %s\n", hash);
+		printf("Manifest state hash:      %s\n", ccr->mfts_hash);
 		printf("Manifest last update:     %s\n",
 		    time2str(ccr->most_recent_update));
-		printf("Manifest references:\n");
+		printf("Manifest instances:\n");
 	}
-	free(hash);
 
 	RB_FOREACH(ccr_mft, ccr_mft_tree, &ccr->mfts) {
 		if (base64_encode(ccr_mft->hash, SHA256_DIGEST_LENGTH, &hash)
@@ -913,48 +880,72 @@ print_ccr_mftstate(struct ccr *ccr)
 		aki = hex_encode(ccr_mft->aki, SHA_DIGEST_LENGTH);
 
 		if (outformats & FORMAT_JSON) {
-			json_do_object("ref", 1);
+			json_do_object("mft_instance", 1);
+
 			json_do_string("hash", hash);
 			json_do_uint("size", ccr_mft->size);
 			json_do_string("aki", aki);
 			json_do_string("seqnum", ccr_mft->seqnum);
 			json_do_int("thisupdate", ccr_mft->thisupdate);
 			json_do_string("sia", ccr_mft->sia);
-			json_do_end();
+
+			json_do_array("subordinates");
+			SLIST_FOREACH(sub, &ccr_mft->subordinates, entry) {
+				ski = hex_encode(sub->ski, SHA_DIGEST_LENGTH);
+				json_do_string("ski", ski);
+				free(ski);
+				ski = NULL;
+			}
+			json_do_end(); /* subordinates */
+
+			json_do_end(); /* mft_instance */
 		} else {
+			int i;
+
 			printf("%26shash:%s size:%zu aki:%s seqnum:%s "
-			    "thisupdate:%lld sia:%s\n", "", hash,
+			    "thisupdate:%lld sia:%s", "", hash,
 			    ccr_mft->size, aki, ccr_mft->seqnum,
 			    (long long)ccr_mft->thisupdate, ccr_mft->sia);
+
+			i = 0;
+			SLIST_FOREACH(sub, &ccr_mft->subordinates, entry) {
+				if (i++ == 0)
+					printf(" subordinates:");
+				else
+					printf(",");
+
+				ski = hex_encode(sub->ski, SHA_DIGEST_LENGTH);
+				printf("%s", ski);
+				free(ski);
+				ski = NULL;
+			}
+
+			printf("\n");
 		}
 
 		free(aki);
 		free(hash);
 	}
 	if (outformats & FORMAT_JSON) {
-		json_do_end();
-		json_do_end();
+		json_do_end(); /* mft_instances */
+		json_do_end(); /* manifest_state */
 	}
 }
 
 static void
 print_ccr_roastate(struct ccr *ccr)
 {
-	char buf[64], *hash;
+	char buf[64];
 	struct vrp *vrp;
 
-	if (base64_encode(ccr->vrps_hash, SHA256_DIGEST_LENGTH, &hash) == -1)
-		errx(1, "base64_encode");
-
 	if (outformats & FORMAT_JSON) {
 		json_do_object("roapayload_state", 0);
-		json_do_string("hash", hash);
+		json_do_string("hash", ccr->vrps_hash);
 		json_do_array("vrps");
 	} else {
-		printf("ROA payload state hash:   %s\n", hash);
+		printf("ROA payload state hash:   %s\n", ccr->vrps_hash);
 		printf("ROA payload entries:\n");
 	}
-	free(hash);
 
 	RB_FOREACH(vrp, ccr_vrp_tree, &ccr->vrps) {
 		ip_addr_print(&vrp->addr, vrp->afi, buf, sizeof(buf));
@@ -983,22 +974,17 @@ print_ccr_roastate(struct ccr *ccr)
 static void
 print_ccr_aspastate(struct ccr *ccr)
 {
-	char *hash;
 	struct vap *vap;
 	size_t i;
 
-	if (base64_encode(ccr->vaps_hash, SHA256_DIGEST_LENGTH, &hash) == -1)
-		errx(1, "base64_encode");
-
 	if (outformats & FORMAT_JSON) {
 		json_do_object("aspapayload_state", 0);
-		json_do_string("hash", hash);
+		json_do_string("hash", ccr->vaps_hash);
 		json_do_array("vaps");
 	} else {
-		printf("ASPA payload state hash:  %s\n", hash);
+		printf("ASPA payload state hash:  %s\n", ccr->vaps_hash);
 		printf("ASPA payload entries:\n");
 	}
-	free(hash);
 
 	RB_FOREACH(vap, vap_tree, &ccr->vaps) {
 		if (outformats & FORMAT_JSON) {
@@ -1035,24 +1021,19 @@ print_ccr_aspastate(struct ccr *ccr)
 static void
 print_ccr_tastate(struct ccr *ccr)
 {
-	char *hash, *ski;
+	char *ski;
 	struct ccr_tas_ski *cts;
 	int i = 0;
 
-	if (base64_encode(ccr->tas_hash, SHA256_DIGEST_LENGTH, &hash) == -1)
-		errx(1, "base64_encode");
-
 	if (outformats & FORMAT_JSON) {
 		json_do_object("trustanchor_state", 0);
-		json_do_string("hash", hash);
+		json_do_string("hash", ccr->tas_hash);
 		json_do_array("skis");
 	} else {
-		printf("Trust anchor state hash:  %s\n", hash);
+		printf("Trust anchor state hash:  %s\n", ccr->tas_hash);
 		printf("Trust anchor keyids:      ");
 	}
 
-	free(hash);
-
 	RB_FOREACH(cts, ccr_tas_tree, &ccr->tas) {
 		ski = hex_encode(cts->keyid, sizeof(cts->keyid));
 
@@ -1078,15 +1059,11 @@ print_ccr_tastate(struct ccr *ccr)
 static void
 print_ccr_rkstate(struct ccr *ccr)
 {
-	char *hash;
 	struct brk *brk;
 
-	if (base64_encode(ccr->brks_hash, SHA256_DIGEST_LENGTH, &hash) == -1)
-		errx(1, "base64_encode");
-
 	if (outformats & FORMAT_JSON) {
 		json_do_object("routerkey_state", 0);
-		json_do_string("hash", hash);
+		json_do_string("hash", ccr->brks_hash);
 		json_do_array("routerkeys");
 		RB_FOREACH(brk, brk_tree, &ccr->brks) {
 			json_do_object("brk", 0);
@@ -1098,7 +1075,7 @@ print_ccr_rkstate(struct ccr *ccr)
 		json_do_end(); /* routerkeys */
 		json_do_end(); /* routerkey_state */
 	} else {
-		printf("Router key state hash:    %s\n", hash);
+		printf("Router key state hash:    %s\n", ccr->brks_hash);
 		printf("Router keys:\n");
 		RB_FOREACH(brk, brk_tree, &ccr->brks) {
 			printf("%26s", "");
@@ -1107,8 +1084,6 @@ print_ccr_rkstate(struct ccr *ccr)
 			printf("pubkey:%s\n", brk->pubkey);
 		}
 	}
-
-	free(hash);
 }
 
 void
